import * as React from 'react';
import { connect, ConnectedProps } from "react-redux";
import { changeCpuUsageLimit } from '../redux/actions';
import * as CpuUsageLimitTypes from '../redux/cpu-usage-limit-types';
import { withTranslation } from 'react-i18next';
import { TFunction } from 'i18next';

const connector = connect(
  (state: RootState) => (
    {
      cpuUsageLimitType: state.conversions.conversion.cpuUsageLimitType,
    }
  ),
  {
    changeCpuUsageLimit
  }
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  t: TFunction;
};

class CpuLimitSelector extends React.Component<Props, {}> {
  constructor(props: Readonly<Props>){
    super(props);
    this.changeCpuUsageLimit = this.changeCpuUsageLimit.bind(this);
  }

  changeCpuUsageLimit(event: React.ChangeEvent<HTMLSelectElement>): void{
    this.props.changeCpuUsageLimit(event.target.value);
  }

  render(): JSX.Element {
    return (
      <div className="cpu-usage-selector">
        <p>{this.props.t('conversion speed')}</p>
        <select value={this.props.cpuUsageLimitType} onChange={this.changeCpuUsageLimit}>
          <option value={CpuUsageLimitTypes.FULL}>{this.props.t('full')}</option>
          <option value={CpuUsageLimitTypes.HALF}>{this.props.t('medium')}</option>
          <option value={CpuUsageLimitTypes.MINIMUM}>{this.props.t('minimum')}</option>
        </select>
      </div>
    );
  }
}

export default withTranslation()(connector(CpuLimitSelector));
