import * as React from 'react';
import { connect, ConnectedProps } from "react-redux";
import * as TransitionableTypes from '../redux/transitionable-types'
import { withTranslation } from 'react-i18next';
import { TFunction } from 'i18next';

const connector = connect(
  (state: RootState) => (
    {
      currentType: state.conversions.conversion.currentType,
    }
  ),
  null,
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  onClickWhenConvertStartable(): void;
  onClickWhenCancelConvertStartable(): void;
  onClickWhenDoneConvert(): void;
  t: TFunction;
};

class DirectoryChooseButton extends React.Component<Props, {}> {
  constructor(props: Readonly<Props>){
    super(props);
    this.onClick = this.onClick.bind(this);
    this.getDisabled = this.getDisabled.bind(this);
    this.getText = this.getText.bind(this);
    this.getColor = this.getColor.bind(this);
  }

  isConvertStartable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_CONVERT.includes(this.props.currentType);
  }

  isInitialTypeState(): boolean{
    return TransitionableTypes.INITIAL_STATE.includes(this.props.currentType);
  }

  isCancelConvertStartable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_CANCEL_CONVERT.includes(this.props.currentType);
  }

  isNextConvertStartable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_NEXT_CONVERT.includes(this.props.currentType);
  }

  isConvertCancelStarting(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_DONE_CANCEL_CONVERT.includes(this.props.currentType);
  }

  isNextConvertStartableWithCanceled(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_NEXT_CONVERT_WITH_CANCELED.includes(this.props.currentType);
  }

  onClick(): void{
    if(this.isConvertStartable()){
      this.props.onClickWhenConvertStartable();
    }else if(this.isCancelConvertStartable()){
      this.props.onClickWhenCancelConvertStartable();
    }else if(this.isNextConvertStartable()){
      this.props.onClickWhenDoneConvert();
    }
  }

  getDisabled(): string{
    if(this.isInitialTypeState() || this.isNextConvertStartableWithCanceled()){
      return 'disabled';
    }else{
      return '';
    }
  }

  getText(): string{
    if(this.isConvertStartable() || this.isInitialTypeState()){
      return this.props.t('choose the output location');
    }else if(this.isCancelConvertStartable() || this.isConvertCancelStarting() || this.isNextConvertStartableWithCanceled()){
      return this.props.t('abort');
    }else if(this.isNextConvertStartable()){
      return this.props.t('open the folder');
    }else{
      return '';
    }
  }

  getColor(): string{
    if(this.isConvertStartable() || this.isInitialTypeState()){
      return 'cyan lighten-1';
    }else if(this.isCancelConvertStartable() || this.isConvertCancelStarting() || this.isNextConvertStartableWithCanceled()){
      return 'red lighten-1';
    }else if(this.isNextConvertStartable()){
      return 'green'
    }else{
      return '';
    }
  }

  render(): JSX.Element {
    return (
      <div className={`btn btn-small choose-dir ${this.getColor()} ${this.getDisabled()}`} onClick={this.onClick}>{this.getText()}</div>
    );
  }
}

export default withTranslation()(connector(DirectoryChooseButton));
