import * as React from 'react';
import { connect, ConnectedProps } from "react-redux";
import * as TransitionableTypes from '../redux/transitionable-types'

const connector = connect(
  (state: RootState) => (
    {
      currentType: state.conversions.conversion.currentType,
    }
  ),
  null,
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  onClickWhenFileChoosable(): void;
  children: React.ReactNode;
};

class FileChooseButton extends React.Component<Props, {}> {
  constructor(props: Readonly<Props>){
    super(props);
    this.onClick = this.onClick.bind(this);
    this.getDisabled = this.getDisabled.bind(this);
  }

  isFileChoosable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_CHOOSE_FILE.includes(this.props.currentType)
  }

  getDisabled(): string{
    if(this.isFileChoosable()){
      return ''
    }else{
      return 'disabled';
    }
  }

  onClick(): void{
    if(this.isFileChoosable()){
      this.props.onClickWhenFileChoosable();
    }
  }

  render(): JSX.Element {
    return (
      <div className={`btn btn-large button cyan lighten-1 choose-file ${this.getDisabled()}`} onClick={this.onClick}>{this.props.children}</div>
    );
  }
}

export default connector(FileChooseButton);
