import * as React from 'react';
import { connect, ConnectedProps } from "react-redux";
import FileChooseButton from './file-choose-button';
import { chooseFile } from '../redux/actions';

const connector = connect(
  (state: RootState) => (
    {
      selectedFilePaths: state.conversions.conversion.selectedFilePaths,
    }
  ),
  { chooseFile }
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  children: React.ReactNode;
};

class FileChooser extends React.Component<Props, {}> {
  ipcRenderer: Electron.IpcRenderer;

  constructor(props: Readonly<Props>){
    super(props);
    this.ipcRenderer = window.Electron.ipcRenderer;
    this.chooseFile = this.chooseFile.bind(this);
  }

  chooseFile(): void{
    this.ipcRenderer.invoke('open-file-dialog').then((choseFile: ChoseFile) => {
      if(!choseFile.canceled){
        this.props.chooseFile(choseFile.filePaths);
      }
    })
  }

  render(): JSX.Element {
    return (
      <FileChooseButton onClickWhenFileChoosable={this.chooseFile}>{this.props.children}</FileChooseButton>
    );
  }
}

export default connector(FileChooser);
