import * as React from 'react';
import { connect, ConnectedProps } from "react-redux";
import * as TransitionableTypes from '../redux/transitionable-types'
import { withTranslation } from 'react-i18next';
import { TFunction } from 'i18next';

const connector = connect(
  (state: RootState) => (
    {
      currentType: state.conversions.conversion.currentType,
      currentFileIndex: state.conversions.conversion.currentFileIndex,
      progress: state.conversions.conversion.progress,
      selectedFilePaths: state.conversions.conversion.selectedFilePaths,
    }
  ),
  null
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  onClickWhenConvertStartable: () => void;
  t: TFunction;
};

class ProgressButton extends React.Component<Props, {}> {
  constructor(props: Readonly<Props>){
    super(props);
    this.onClick = this.onClick.bind(this);
    this.getColor = this.getColor.bind(this);
    this.getDisabled = this.getDisabled.bind(this);
    this.getText = this.getText.bind(this);
  }

  isConvertStartable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_CONVERT.includes(this.props.currentType);
  }

  isProgressUpdatable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_UPDATE_PROGRESS.includes(this.props.currentType);
  }

  isNextConvertStartable(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_NEXT_CONVERT.includes(this.props.currentType);
  }

  isNextConvertStartableWithCanceled(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_START_NEXT_CONVERT_WITH_CANCELED.includes(this.props.currentType);
  }

  isConvertCancelStarting(): boolean{
    return TransitionableTypes.TRANSITIONABLE_TO_DONE_CANCEL_CONVERT.includes(this.props.currentType);
  }

  isInitialState(): boolean{
    return TransitionableTypes.INITIAL_STATE.includes(this.props.currentType);
  }

  onClick(): void{
    if(this.isConvertStartable()){
      this.props.onClickWhenConvertStartable();
    }
  }

  getColor(): string{
    if(this.isNextConvertStartableWithCanceled() || this.isConvertCancelStarting()){
      return 'red darken-1';
    }else if(this.isNextConvertStartable()){
      return 'cyan darken-1';
    }else if(this.isConvertStartable()){
      return 'cyan lighten-1';
    }else if(this.isInitialState()){
      return 'cyan lighten-1';
    }else if(this.isProgressUpdatable()){
      return 'cyan lighten-1';
    }else{
      return '';
    }
  }

  getText(): string{
    if(this.isConvertCancelStarting()){
      return this.props.t('aborting');
    }else if(this.isNextConvertStartableWithCanceled()){
      return this.props.t('aborted');
    }else if(this.isNextConvertStartable()){
      return this.props.t('done');
    } else if(this.isConvertStartable() || this.isInitialState()){
      return this.props.t('convert');
    } else if(this.isProgressUpdatable()){
      return `${this.props.t('converting')} ${this.props.progress.toFixed(2)}% ${(this.props.currentFileIndex + 1)} / ${this.props.selectedFilePaths.length}`;
    } else {
      return '';
    }
  }

  getDisabled(): string{
    return this.isInitialState() ? 'disabled' : '';
  }

  render(): JSX.Element {
    return (
      <div className={`btn btn-large button progress-button choose-file ${this.getColor()} ${this.getDisabled()}`} onClick={this.onClick}>
        <div className="progress-button-progress cyan darken-1" style={{width: Math.max(0,this.props.progress) + '%'}}>
        </div>
        <div className="progress-button-text">
          {this.getText()}
        </div>
      </div>
    );
  }
}

export default withTranslation()(connector(ProgressButton));
