import * as React from 'react';
import ProgressButton from '../atoms/progress-button'
import { connect, ConnectedProps } from "react-redux";
import { chooseDirectory, updateProgress, completeConvert, startConvert, startCancelConvert, doneCancelConvert } from '../redux/actions';
import DirChooseButton from '../atoms/dir-choose-button';
import { withTranslation } from 'react-i18next';
import { TFunction } from 'i18next';

const connector = connect(
  (state: RootState) => (
    {
      currentType: state.conversions.conversion.currentType,
      currentFileIndex: state.conversions.conversion.currentFileIndex,
      progress: state.conversions.conversion.progress,
      selectedFilePaths: state.conversions.conversion.selectedFilePaths,
      convertDestinationPath: state.conversions.conversion.convertDestinationPath,
      cpuUsageLimitType: state.conversions.conversion.cpuUsageLimitType,
    }
  ),
  {
    chooseDirectory,
    startConvert,
    updateProgress,
    completeConvert,
    startCancelConvert,
    doneCancelConvert
  }
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
  t: TFunction;
};

class Converter extends React.Component<Props> {
  ipcRenderer: Electron.IpcRenderer;

  constructor(props: Readonly<Props>){
    super(props);
    this.ipcRenderer = window.Electron.ipcRenderer;

    this.ipcRenderer.on('progress-update', (_, progress: number) => {
      this.props.updateProgress(this.props.currentFileIndex, progress)
    });

    this.startConvert = this.startConvert.bind(this);
    this.chooseDestinationPath =  this.chooseDestinationPath.bind(this);
    this.stopConvert = this.stopConvert.bind(this);
    this.openDir = this.openDir.bind(this);
  }

  chooseDestinationPath(): void {
    this.ipcRenderer.invoke('open-dir-dialog').then((choseDir: ChoseFile) => {
      if(!choseDir.canceled){
        this.props.chooseDirectory(choseDir.filePaths[0]);
      }
    })
  }

  async startConvert(): Promise<void> {
    this.props.startConvert();
    const pathDelimiter = process.platform === 'win32' ? '\\' : '/';
    const destinationPathArray = this.props.convertDestinationPath.split(pathDelimiter);
    for(let i = 0; i<this.props.selectedFilePaths.length; i++){
      this.props.updateProgress(i, 0);
      const mediaDimension: number[] = await this.ipcRenderer.invoke('ffprobe-exec', this.props.selectedFilePaths[i].split(pathDelimiter), destinationPathArray);
      await this.ipcRenderer.invoke('ffmpeg-exec', this.props.selectedFilePaths[i].split(pathDelimiter), destinationPathArray, mediaDimension, this.props.cpuUsageLimitType);
    }
    this.props.completeConvert();
  }

  async stopConvert(): Promise<void>{
    this.props.startCancelConvert();
    await this.ipcRenderer.invoke('ffprobe-kill');
    await this.ipcRenderer.invoke('ffmpeg-kill');
    this.props.doneCancelConvert();
  }

  openDir(): void{
    const pathDelimiter = process.platform === 'win32' ? '\\' : '/';
    const destinationPathArray = (this.props.convertDestinationPath).split(pathDelimiter);
    this.ipcRenderer.invoke('oepn-dir', destinationPathArray);
  }

  render(): JSX.Element {
    return (
      <div className="column converter-column">
        <div className="teal lighten-1 circle">2</div>
        <ProgressButton onClickWhenConvertStartable={this.startConvert}/>
        <div className="file-output-row">
          <div className="file-output">
            <p className="file-output-text">{this.props.t('output location')}</p>
            <input className="dir-name" type="text" readOnly value={this.props.convertDestinationPath}/>
          </div>
          <div className="file-output-dir-chooser">
            <DirChooseButton onClickWhenConvertStartable={this.chooseDestinationPath} onClickWhenCancelConvertStartable={this.stopConvert} onClickWhenDoneConvert={this.openDir}/>
          </div>
        </div>
      </div>
    );
  }
}

export default withTranslation()(connector(Converter));
