import Header from '../organisms/header'
import Footer from '../organisms/footer'
import Main from '../organisms/main'
import * as React from 'react'

import { chooseFile } from '../redux/actions';
import { connect, ConnectedProps } from "react-redux";

const connector = connect(
  (state: RootState) => (
    {
      selectedFilePaths: state.conversions.conversion.selectedFilePaths,
    }
  ),
  { chooseFile }
)

type PropsFromRedux = ConnectedProps<typeof connector>

type Props = PropsFromRedux & {
};

class Index extends React.Component<Props, {dragover: boolean}> {
  constructor(props: Readonly<Props>) {
    super(props);
    this.state = {dragover: false};
    this.drop = this.drop.bind(this);
    this.dragStart = this.dragStart.bind(this);
    this.dragEnd = this.dragEnd.bind(this);
  }

  drop(event: React.DragEvent<HTMLDivElement>): void{
    event.preventDefault();
    event.stopPropagation();
    const files = [];
    for(let i=0;i<event.dataTransfer.files.length;i++){
      files.push(event.dataTransfer.files[i].path);
    }
    this.props.chooseFile(files);
    this.setState({
      dragover: false
    })
  }

  dragStart(event: React.DragEvent<HTMLDivElement>): void{
    event.preventDefault();
    event.stopPropagation();
    this.setState({
      dragover: true
    })
  }

  dragEnd(event: React.DragEvent<HTMLDivElement>): void{
    event.preventDefault();
    event.stopPropagation();
    this.setState({
      dragover: false
    })
  }

  render(): JSX.Element {
    return (
      <div className={`root ${this.state.dragover ? ' cyan lighten-4' : ''}`} onDrop={this.drop} onDragOver={this.dragStart} onDragEnter={this.dragStart} onDragEnd={this.dragEnd} onDragLeave={this.dragEnd}>
        <Header />
        <Main />
        <Footer />
      </div>
    );
  }
}

export default connector(Index);
