import * as ActionTypes from '../action-types'
import * as TransitionableStates from '../transitionable-types'
import getDestinationPath from '../logics/get-destination-path';
import filterFileTypes from '../logics/filter-file-types';
import * as CpuUsageLimitTypes from '../cpu-usage-limit-types';


const initialState: ConversionState = {
  conversion: {
    currentType: ActionTypes.INITIAL,
    selectedFilePaths: [],
    currentFileIndex: -1,
    convertDestinationPath: '',
    progress: -1,
    cpuUsageLimitType: CpuUsageLimitTypes.HALF,
  }
}

const reducer = (
  state: ConversionState = initialState,
  action: ConversionAction
): ConversionState => {
  switch (action.type) {
    case ActionTypes.CHOOSE_FILE:
      if(TransitionableStates.TRANSITIONABLE_TO_CHOOSE_FILE.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        const filteredFilePaths = filterFileTypes(action.conversion.selectedFilePaths);
        if(filteredFilePaths.length > 0){
          newState.conversion.currentType = action.type;
          newState.conversion.selectedFilePaths = filteredFilePaths;
          newState.conversion.convertDestinationPath = getDestinationPath(newState.conversion.selectedFilePaths);
          newState.conversion.currentFileIndex = -1;
          newState.conversion.progress = -1;
        }
        return newState;
      }
      break;
    case ActionTypes.CHOOSE_DIRECTORY:
      if(TransitionableStates.TRANSITIONABLE_TO_START_CONVERT.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.convertDestinationPath = action.conversion.convertDestinationPath;
        return newState;
      }
      break;
    case ActionTypes.START_CONVERT:
      if(TransitionableStates.TRANSITIONABLE_TO_START_CONVERT.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.currentType = action.type;
        newState.conversion.currentFileIndex = 0;
        newState.conversion.progress = 0;
        return newState;
      }
      break;
    case ActionTypes.UPDATE_PROGRESS:
      if(TransitionableStates.TRANSITIONABLE_TO_UPDATE_PROGRESS.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.currentType = action.type;
        newState.conversion.currentFileIndex = action.conversion.currentFileIndex;
        newState.conversion.progress = action.conversion.progress;
        return newState;
      }
      break;
    case ActionTypes.COMPLETE_CONVERT:
      if(TransitionableStates.TRANSITIONABLE_TO_COMPLETE_CONVERT.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.currentType = action.type;
        newState.conversion.selectedFilePaths = [];
        newState.conversion.currentFileIndex = -1;
        newState.conversion.progress = -1;
        return newState;
      }
      break;
    case ActionTypes.START_CANCEL_CONVERT:
      if(TransitionableStates.TRANSITIONABLE_TO_START_CANCEL_CONVERT.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.currentType = action.type;
        newState.conversion.progress = 0;
        return newState;
      }
      break;
    case ActionTypes.DONE_CANCEL_CONVERT:
      if(TransitionableStates.TRANSITIONABLE_TO_DONE_CANCEL_CONVERT.includes(state.conversion.currentType)){
        const newState = {
          ...state
        };
        newState.conversion.currentType = action.type;
        newState.conversion.selectedFilePaths = [];
        newState.conversion.currentFileIndex = -1;
        newState.conversion.progress = -1;
        return newState;
      }
      break;
    case ActionTypes.CHANGE_CPU_USAGE_LIMIT:
      {
        const newState = {
          ...state
        };
        newState.conversion.cpuUsageLimitType = action.conversion.cpuUsageLimitType;
        return newState;
      }
  }
  return state
}

export default reducer
